@@ -1,38 +1,40 @@
-#' Download WFS layer
+#' @title Download WFS layer
 #'
-#' Download a shapefile layer from IGN Web Feature Service (WFS).
-#'  To do that, it need a location giving by a shape, an apikey
+#' @description
+#' Read simple features from IGN Web Feature Service (WFS).
+#' Three minimal info are needed : a location, an apikey
 #' and the name of layer. You can find those information from
 #' [IGN website](https://geoservices.ign.fr/services-web-experts)
 #'
 #' @usage
-#' get_wfs(shape = NULL,
+#' get_wfs(x = NULL,
 #'         apikey = NULL,
-#'         layer_name = NULL,
+#'         layer = NULL,
 #'         filename = NULL,
 #'         spatial_filter = "bbox",
 #'         ecql_filter = NULL,
 #'         overwrite = FALSE,
 #'         interactive = FALSE)
 #'
-#' @param shape Object of class `sf`. Needs to be located in
+#' @param x Object of class `sf` or `sfc`. Needs to be located in
 #' France.
-#' @param apikey API key from `get_apikeys()` or directly
+#' @param apikey `character`; API key from `get_apikeys()` or directly
 #' from [IGN website](https://geoservices.ign.fr/services-web-experts)
-#' @param layer_name Name of the layer from `get_layers_metadata(apikey, "wfs")`
+#' @param layer `character`; name of the layer from `get_layers_metadata(apikey, "wfs")`
 #' or directly from
 #' [IGN website](https://geoservices.ign.fr/services-web-experts)
 #' @param filename Either a character string naming a file or a connection open
 #' for writing. (ex : "test.shp" or "~/test.shp")
-#' @param spatial_filter Character corresponding to a spatial predicate from ECQL language.
+#' @param spatial_filter `character`; spatial predicate from ECQL language.
 #' See detail and examples for more info.
-#' @param ecql_filter Character corresponding to an ECQL query. See detail and examples
-#' for more info.
-#' @param overwrite If TRUE, file is overwrite
-#' @param interactive if set to TRUE, no need to specify `apikey` and `layer_name`, you'll be ask.
+#' @param ecql_filter `character`; corresponding to an ECQL query.
+#' See detail and examples for more info.
+#' @param overwrite `logical`; if TRUE, file is overwrite.
+#' @param interactive `character`; if TRUE, no need to specify `apikey`
+#' and `layer`, you'll be ask.
 #'
 #' @return
-#' `get_wfs`return an object of class `sf`
+#' `sf` object from `sf` package or `NULL` if no data.
 #'
 #' @export
 #'
@@ -49,10 +51,11 @@
 #' spatial_filter = c("dwithin", distance, units). More info about ECQL language
 #' [here](https://docs.geoserver.org/latest/en/user/filter/ecql_reference.html).
 #' Be aware that "dwithin" is broken and it doesn't accept units properly. Only degrees can be used.
-#' To avoid this, I recommend to use compute a buffer and use "within" instead od "dwithin".
+#' To avoid this, create a buffer and then use "within" instead od "dwithin".
 #' * ECQL query can be provided to `ecql_filter`. This allows direct query of the IGN's WFS
-#' geoservers. If `shape` is set, then the `ecql_filter` comes in addition to the
-#' `spatial_filter`. More info for writing ECQL [here](https://docs.geoserver.org/latest/en/user/tutorials/cql/cql_tutorial.html)
+#' geoservers. If `x` is set, then the `ecql_filter` comes in addition to the
+#' `spatial_filter`. More info for writing ECQL
+#' [here](https://docs.geoserver.org/latest/en/user/tutorials/cql/cql_tutorial.html)
 #'
 #' @seealso
 #' [get_apikeys()], [get_layers_metadata()]
@@ -62,81 +65,101 @@
 #' library(sf)
 #' library(tmap)
 #'
-#' # shape from the best town in France
+#' # Shape from the best town in France
 #' penmarch <- read_sf(system.file("extdata/penmarch.shp", package = "happign"))
 #'
 #' # For quick testing, use interactive = TRUE
-#' shape <- get_wfs(shape = penmarch,
+#' shape <- get_wfs(x = penmarch,
 #'                  interactive = TRUE)
 #'
-#' # For specific use, choose apikey with get_apikey() and layer_name with get_layers_metadata()
+#' # For specific use, choose apikey with get_apikey() and layer with get_layers_metadata()
 #' ## Getting borders of best town in France
 #' apikey <- get_apikeys()[1]
 #' metadata_table <- get_layers_metadata(apikey, "wfs")
-#' layer_name <- metadata_table[32,1]
+#' layer <- metadata_table[32,1] # LIMITES_ADMINISTRATIVES_EXPRESS.LATEST:commune
 #'
 #' # Downloading borders
-#' borders <- get_wfs(penmarch, apikey, layer_name)
+#' borders <- get_wfs(penmarch, apikey, layer)
 #'
 #' # Plotting result
 #' qtm(borders, fill = NULL, borders = "firebrick") # easy map
 #'
 #' # Get forest_area of the best town in France
-#' forest_area <- get_wfs(shape = borders,
+#' forest_area <- get_wfs(x = borders,
 #'                        apikey = "environnement",
-#'                        layer_name = "LANDCOVER.FORESTINVENTORY.V1:resu_bdv1_shape")
+#'                        layer = "LANDCOVER.FORESTINVENTORY.V1:resu_bdv1_shape")
 #'
 #' qtm(forest_area, fill = "libelle")
 #'
-#' # using ECQL filters to query IGN server
+#' # Using ECQL filters to query IGN server
+#' ## First find attributes of the layer
+#' attrs <- get_wfs_attributes(apikey, layer)
 #'
-#' # find all commune's name starting by "plou".
-#' # First you need the name of the attribute to filter
-#' names(borders) # In our case "nom_m" is what we need
-#'
-#' attribute_name <- names(get_wfs(penmarch,apikey, layer_name))
-#' plou_borders <- get_wfs(shape = NULL, # When shape is NULL, all France is query
+#' ## e.g. : find all commune's name starting by "plou"
+#' plou_borders <- get_wfs(x = NULL, # When x is NULL, all France is query
 #'                         apikey = "administratif",
-#'                         layer_name = "LIMITES_ADMINISTRATIVES_EXPRESS.LATEST:commune",
+#'                         layer = "LIMITES_ADMINISTRATIVES_EXPRESS.LATEST:commune",
 #'                         ecql_filter = "nom_m LIKE 'PLOU%'")
+#' qtm(plou_borders)
 #'
-#' # it's also possible to combine ecql_filter
-#' plou_borders <- get_wfs(shape = NULL, # When shape is NULL, all France is query
-#'                         apikey = "administratif",
-#'                         layer_name = "LIMITES_ADMINISTRATIVES_EXPRESS.LATEST:commune",
-#'                         ecql_filter = "nom_m LIKE 'PLOU%' AND population < 2000")
-#'
-#'
+#' ## Combining ecql_filters
+#' plou_borders_inf_2000 <- get_wfs(x = NULL, # When x is NULL, all France is query
+#'                                  apikey = "administratif",
+#'                                  layer = "LIMITES_ADMINISTRATIVES_EXPRESS.LATEST:commune",
+#'                                  ecql_filter = "nom_m LIKE 'PLOU%' AND population < 2000")
+#' qtm(plou_borders)+ qtm(plou_borders_inf_2000, fill = "red")
 #' }
-get_wfs <- function(shape = NULL,
+get_wfs <- function(x = NULL,
                     apikey = NULL,
-                    layer_name = NULL,
+                    layer = NULL,
                     filename = NULL,
                     spatial_filter = "bbox",
                     ecql_filter = NULL,
                     overwrite = FALSE,
                     interactive = FALSE){
 
-   check_get_wfs_input(shape, spatial_filter)
-   if (!is.null(shape)){shape <- st_make_valid(shape)}
+   # check input ----
+   ## x
+   if (!inherits(x, c("sf", "sfc", "NULL"))){
+      stop("`x` should have class `sf`, `sfc` or `NULL` if `ecql_filter` is set.",
+           call. = FALSE)
+   }
 
-   # interactively hoose apikey and layer_name
+   if (!is.null(x)){
+      x <- st_make_valid(x)
+   }
+
+   ## spatial_filter
+   if(!is.null(spatial_filter)){
+      spatial_predicate <- c("INTERSECTS", "DISJOINT", "CONTAINS", "WITHIN", "TOUCHES", "BBOX",
+                             "CROSSES", "OVERLAPS", "EQUALS", "RELATE", "DWITHIN", "BEYOND")
+
+      is_valid_predicate <- is.element(toupper(spatial_filter[1]), spatial_predicate)
+      if (!is_valid_predicate){
+         stop("`spatial_filter` should be one of : ",
+              paste0(spatial_predicate, collapse =", "), call. = FALSE)
+      }
+   }
+
+
+   # interactive session ----
    if (interactive){
       apikeys <- get_apikeys()
       apikey <- apikeys[menu(apikeys)]
 
       layers <- get_layers_metadata(apikey, data_type = "wfs")$Name
-      layer_name <- layers[menu(layers)]
+      layer <- layers[menu(layers)]
    }
 
+   # prep url and hit api ----
    # When spatial filter "bbox" isn't used, crs are needed
    is_bbox <- sum(spatial_filter == "bbox") == 1
    if(!is_bbox){
-      crs <- get_wfs_default_crs(apikey, layer_name)
+      crs <- get_wfs_default_crs(apikey, layer)
    }
 
    # hit api and loop if there more than 1000 features
-   req <- build_wfs_req(shape, apikey, layer_name, spatial_filter,
+   req <- build_wfs_req(x, apikey, layer, spatial_filter,
                         ecql_filter, startindex = 0, crs)
    resp <- hit_api_wfs(req, ecql_filter, apikey)
 
@@ -152,7 +175,7 @@ get_wfs <- function(shape = NULL,
    temp <- resp
    while(nrow(temp) == 1000){
       message("...", appendLF = F)
-      url <- build_wfs_req(shape, apikey, layer_name, spatial_filter,
+      url <- build_wfs_req(x, apikey, layer, spatial_filter,
                            ecql_filter, startindex = i, crs)
       temp <- hit_api_wfs(url, ecql_filter, apikey)
       resp <- rbind(resp, temp)
@@ -160,6 +183,8 @@ get_wfs <- function(shape = NULL,
       i <- i + 1000
    }
    cat("\n")
+
+   # post treatment ----
    # Cleaning list column from features
    resp <- resp[ , !sapply(resp, is.list)]
 
@@ -172,30 +197,32 @@ get_wfs <- function(shape = NULL,
    return(resp)
 }
 
-#' construct url
-#' @param shape Object of class `sf`. Needs to be located in France.
+#' @title build_wfs_req
+#' @description construct url
+#'
+#' @param x Object of class `sf`. Needs to be located in France.
 #' @param apikey API key from `get_apikeys()`
-#' @param layer_name Name of the layer
+#' @param layer Name of the layer
 #' @param spatial_filter See ?get_wfs
 #' @param ecql_filter See ?get_wfs
 #' @param startindex startindex for looping when more than 1000 features are returned
 #' @param crs epsg character from `get_wfs_default_crs`
 #' @noRd
 #'
-build_wfs_req <- function(shape,
+build_wfs_req <- function(x,
                           apikey,
-                          layer_name,
+                          layer,
                           spatial_filter = NULL,
                           ecql_filter = NULL,
                           startindex = 0,
                           crs = NULL){
 
-   shape_exist <- !is.null(shape)
+   shape_exist <- !is.null(x)
    spatial_filter_exist <- !is.null(spatial_filter)
    spatial_predicate <- NULL
 
    if (shape_exist & spatial_filter_exist){
-      spatial_predicate <- construct_spatial_filter(shape, spatial_filter,
+      spatial_predicate <- construct_spatial_filter(x, spatial_filter,
                                                     crs, apikey)
    }
 
@@ -207,7 +234,7 @@ build_wfs_req <- function(shape,
       request = "GetFeature",
       outputFormat = "json",
       srsName = "EPSG:4326",
-      typeName = layer_name,
+      typeName = layer,
       startindex = startindex,
       count = 1000
    )
@@ -242,8 +269,8 @@ hit_api_wfs <- function(req,
             stop("Check that `ecql_filter` is properly set.", call. = F)
          }else{
             stop("Please check that :\n",
-                 "- shape is not empty ;\n",
-                 "- layer_name is valid by running ",
+                 "- x is not empty ;\n",
+                 "- layer is valid by running ",
                  "`get_layers_metadata(\"", apikey, "\", \"wfs\")`\n",
                  call. = F)
          }})
@@ -256,14 +283,15 @@ hit_api_wfs <- function(req,
 #' for writing. (ex : "test.shp" or "~/test.shp")
 #' @param resp response from hit_api_wfs request
 #' @param overwrite If TRUE, file is overwrite
+#' @param quiet see ?sf::st_write
 #' @noRd
-save_wfs <- function(filename, resp, overwrite){
+save_wfs <- function(filename, resp, overwrite, quiet = F){
 
    path <- normalizePath(filename, mustWork = FALSE)
    path <- enc2utf8(path)
 
    tryCatch({
-      st_write(resp, path, delete_dsn = overwrite)
+      st_write(resp, path, delete_dsn = overwrite, quiet = quiet)
    },
    error = function(cnd){
       if (grepl("Dataset already exists", cnd)){
@@ -274,31 +302,6 @@ save_wfs <- function(filename, resp, overwrite){
    })
 }
 
-#' save check_get_wfs_input
-#' @param shape Object of class `sf` or `sfc`
-#' @param spatial_filter spatial operator from ecql language
-#' @noRd
-check_get_wfs_input <- function(shape, spatial_filter){
-   # Check input parameter
-   ## shape
-   is_sf_or_sfc <- inherits(shape, c("sf", "sfc", "NULL"))
-   if (!is_sf_or_sfc){
-      stop("`shape` should have class `sf`, `sfc` or `NULL` if `ecql_filter` are set.",
-           call. = FALSE)
-   }
-
-   ## spatial_filter
-   if(!is.null(spatial_filter)){
-      spatial_predicate <- c("INTERSECTS", "DISJOINT", "CONTAINS", "WITHIN", "TOUCHES", "BBOX",
-                             "CROSSES", "OVERLAPS", "EQUALS", "RELATE", "DWITHIN", "BEYOND")
-
-      is_valid_predicate <- is.element(toupper(spatial_filter[1]), spatial_predicate)
-      if (!is_valid_predicate){
-         stop("`spatial_filter` should be one of : ",
-              paste0(spatial_predicate, collapse =", "), call. = FALSE)
-      }
-   }
-}
 
 #' @description construct ecql spatial filter string
 #' @param shape object of class sf or sfc
@@ -309,7 +312,7 @@ check_get_wfs_input <- function(shape, spatial_filter){
 #' @return ecql string
 #' @noRd
 #'
-construct_spatial_filter <- function(shape = NULL,
+construct_spatial_filter <- function(x = NULL,
                                      spatial_filter = NULL,
                                      crs = NULL,
                                      apikey = NULL){
@@ -327,14 +330,14 @@ construct_spatial_filter <- function(shape = NULL,
    # particular case for bbox
    is_bbox <- (spatial_filter[1] == "bbox")
    if (is_bbox){
-      bbox <- st_bbox(shape)
+      bbox <- st_bbox(x)
       spatial_filter <- c(spatial_filter,
                           bbox["xmin"], bbox["ymin"], bbox["xmax"], bbox["ymax"],
-                          sprintf("'EPSG:%s'", st_crs(shape)$epsg))
+                          sprintf("'EPSG:%s'", st_crs(x)$epsg))
    }
 
    # if is "bbox", geom is null
-   geom <- switch(is_bbox + 1, st_as_text_happign(shape, crs), NULL)
+   geom <- switch(is_bbox + 1, st_as_text_happign(x, crs), NULL)
 
    # Build final spatial filter
    spatial_filter <- sprintf("%s(%s, %s)",
